
// (c) COPYRIGHT URI/MIT 1994-1996
// Please read the full copyright statement in the file COPYRIGH.  
//
// Authors:
//      dan		Dan Holloway (dholloway@gso.uri.edu)

// Implementation for the class JGSequence. See JGByte.cc
//
// jhrg 1/12/95

// $Log: JGSequence.cc,v $
// Revision 1.11  1998/08/06 16:32:35  jimg
// Fixed misuse of the read(...) member function. Return true if more data
// is to be read, false is if not and error if an error is detected
//
// Revision 1.10  1998/02/19 19:47:18  jimg
// Reverted to old read() member function.
// Removed stuff for the optimized Sequence transfer.
//
// Revision 1.9  1998/02/05 20:14:34  jimg
// DODS now compiles with gcc 2.8.x
//
// Revision 1.8  1997/10/09 22:19:42  jimg
// Resolved conflicts in merge of 2.14c to trunk.
//
// Revision 1.7  1996/11/08 01:01:44  jimg
// Fixed a problem in read() where values were packed to tightly (every 40
// bytes rather than 80). This caused other code to effectively skip every
// other data value.
//
// Revision 1.6  1996/09/17 00:02:01  jimg
// Misc changes associated with merging branch code to the trunk.
//
// Revision 1.5.2.1.2.1  1996/06/26 22:18:02  jimg
// Dans changes for 2.05.
//
// Revision 1.4  1995/10/11  23:06:12  jimg
// These files are the result of merging Dan's changes into the main
// trunk. However, the archive was messed up in the process, so it may be hard
// to trace back to the original versions of some of these files.
//
// Revision 1.3  1995/07/09  21:39:53  jimg
// Added copyright notice.
//
// Revision 1.2  1995/03/14  01:23:31  jimg
// Modified so that these files are a proper subclass of the DAP classes
// Byte, ..., Grid.
// Added assert macros.
// Added Jgofs.h external declarations file.
//
// Revision 1.1  1995/02/16  19:02:31  dan
// JGOFS sub-class definitions for Sequence.cc
//
// Revision 1.2  1995/01/19  21:59:00  jimg
// Added read_val from dummy_read.cc to the sample set of sub-class
// implementations.
// Changed the declaration of readVal in BaseType so that it names the
// mfunc read_val (to be consistant with the other mfunc names).
// Removed the unnecessary duplicate declaration of the abstract virtual
// mfuncs read and (now) read_val from the classes Byte, ... Grid. The
// declaration in BaseType is sufficient along with the decl and definition
// in the *.cc,h files which contain the subclasses for Byte, ..., Grid.
//
// Revision 1.1  1995/01/19  20:20:51  jimg
// Created as an example of subclassing the class hierarchy rooted at
// BaseType.
//

#define DODS_DEBUG 1

#include "config_jg.h"

#ifdef _GNUG_
#pragma implementation
#endif

#include "JGSequence.h"
#include "jgofs.h"

// Note that this global static value must be initialized this way. However,
// since the value is used only on the server-side of DODS, link-time
// problems encountered with global static objects which required
// initialization should not cause us any grief. Also note that this makes
// the servers non-reentrant (which should currently pose no problem). jhrg
// 1/6/98. 
#if 0
int JGSequence::_current_level = 0;
#endif

JGSequence *
NewSequence(const String &n)
{
    return new JGSequence(n);
}

BaseType *
JGSequence::ptr_duplicate()
{
    return new JGSequence(*this);
}

JGSequence::JGSequence(const String &n) : Sequence(n) //, _use_buffer(false)
{
}

JGSequence::~JGSequence()
{
}

bool
JGSequence::read(const String &dataset, int &error)
{
    bool status = true;
    int level;

    // Warning from g++: varValue is a char (*)[80] but the formal param is
    // an fourty element array.

    // I added the following variable and modified the if statement below
    // since varValue is an array of things that are 80 bytes long. However,
    // varSize is set to 40 in jgdds.cc so ljdbreada_() stuffs values in it
    // every 40 chars. When other code goes to read values out it uses the
    // decalred size of the array (80) and thus skips every other value. 
    // Note that the symbol VALSIZE is defined to be `80'.
    // jhrg 11/7/96

    int valSize = VALSIZE;

    if ((level = ljdbreada_(&unit, varValue, &valSize)) >= 0) {

	for (Pix p = first_var(); p; next_var(p)) {
	    if (var(p)->type() == dods_sequence_c) {
		status = read_level(dataset, (JGSequence *)var(p), error);
		var(p)->set_read_p(status);
	    }	
	    else {
		// status = status && var(p)->read(dataset, error);
		// Note: 
		var(p)->read(dataset, error);
		if (error)	// Exit if an error is found
		    return false;
	    }
	}
    }
    else {
	error = 0;		// Reached the end of the Sequence, exit.
	status = false;
    }

    return status;
}

// Make read_level use the same semantics as read; return false for no more
// data, true for more data and set `error' if/when a problem is found.

bool
JGSequence::read_level(const String &dataset, JGSequence *seq, int &error)
{
    bool status = true;

    for (Pix p = seq->first_var(); p; seq->next_var(p)) {
	if (seq->var(p)->type() == dods_sequence_c) {
	    status = read_level(dataset, (JGSequence *)seq->var(p), error);
	    seq->set_read_p(status);
	}
	else { 
	    // status = status && seq->var(p)->read(dataset, error);
	    seq->var(p)->read(dataset, error);
	    if (error)		// stop reading if an error is found
		return false;
	}
    }

    return status;
}

#if 0

void
JGSequence::set_current_level(int cl)
{
    _current_level = cl;
}

int
JGSequence::current_level()
{
    // _current_level is a static class memeber.
    return _current_level;
}

bool
JGSequence::use_buffer_p()
{
    return _use_buffer;
}

void
JGSequence::set_use_buffer_p(bool state)
{
    _use_buffer = state;
    for (Pix p = first_var(); p; next_var(p))
	// Really need rtti here since a dods_sequence_c might not be a
	// JGSequence... jhrg 1/6/98.
	if (var(p)->type() == dods_sequence_c)
	    ((JGSequence *)var(p))->set_use_buffer_p(state);
}

bool
JGSequence::child_sequences_p()
{
    for (Pix p = first_var(); p; next_var(p))
	if (var(p)->type() == dods_sequence_c)
	    return true;

    return false;
}

bool
JGSequence::read(const String &dataset, int &error)
{
    bool status = true;

    DBG(cerr << "current_level = " << current_level() << endl);

    // Test for EOF from jdbread().
    if (current_level() == -1)
	return false;

    if (!use_buffer_p()) {
	int valSize = VALSIZE;
	int lvl = ljdbreada_(&unit, varValue, &valSize);
	DBG(cerr << "Called jdbread(), level = " << lvl << endl);
	set_current_level(lvl);
	set_use_buffer_p(true);
	// Back out of the recursive call
	if (current_level() < level())
	    return true;
    }

    if (!child_sequences_p())
	set_use_buffer_p(false);

    // In the following loop call read for all of the current level's
    // simple-type members. Do nothing for composite types since the
    // Sequence::serialize() mfunc will take care of them (by invoking
    // serialize on them recursively causing read to be called, etc.
    // Note that JGOFS only supports String and Float32 variables but I test
    // for any simple type (and lump String and Url with them).
    for (Pix p = first_var(); p; next_var(p)) {
	switch (var(p)->type()) {
	  case dods_byte_c:
	  case dods_int32_c:
	  case dods_uint32_c:
	  case dods_float64_c:
	  case dods_str_c:
	  case dods_url_c: 
	    // NB: this call to read never invokes JGSequence::read() so this
	    // call never changes the value returned by current_level().
	    status = status && var(p)->read(dataset, error);
	    if (error)
		return false;
	    break;

	  default:
	    break;
	}
    }

    return status;
}
#endif

