
// This file contains functions which read the variables and their attributes
// from a netcdf file and build the in-memeory DAS. These functions form the
// core of the server-side software necessary to extract the DAS from a
// netcdf data file.
//
// It also contains test code which will print the in-memeory DAS to
// stdout. It uses both the DAS class as well as the netcdf library.
// In addition, parts of these functions were taken from the netcdf program
// ncdump, from the netcdf standard distribution (ver 2.3.2)
//
// jhrg 9/23/94

// $Log: jgdas.cc,v $
// Revision 1.11  1998/02/05 20:14:38  jimg
// DODS now compiles with gcc 2.8.x
//
// Revision 1.10  1997/03/10 22:58:24  dan
// Added Error-handling.  Use new API routines added to local
// jgofs library to retrieve errors generated by the local methods.
//
// Revision 1.9  1996/12/18 17:44:38  jimg
// Removed dead code.
// Fixed the prototype of ErrMsgT() and uncommented its used in a conditional
// that tested for an error. All part of the long, continuing, saga of the
// PATH_INFO bug.
//
// Revision 1.8  1996/11/09 00:41:59  jimg
// Increased the number of variable that can be processed from 99 to NVALS
// (250).
//
// Revision 1.7  1996/09/17 00:04:12  jimg
// Misc changes associated with merging branch code to the trunk.
//
// Revision 1.6.2.1.2.1  1996/06/26 22:18:23  jimg
// Dans changes for 2.05.
//
// Revision 1.5  1995/10/11  23:56:10  jimg
// Added external symbols defined/declared in jgofs.h to lgofs/ljdb.c. Fixed
// the C constant TOKEN so that it is `40' everywhere.
//
// Revision 1.4  1995/10/11  23:06:28  jimg
// These files are the result of merging Dan's changes into the main
// trunk. However, the archive was messed up in the process, so it may be hard
// to trace back to the original versions of some of these files.
//
// Revision 1.3.2.1  1995/09/18  22:48:06  jimg
// Merged Dan's changes with the trunk version. This will be commited to the
// release-1-0-patches branch and then merged back into the trunk later.
//
// Revision 1.3  1995/07/09  21:40:04  jimg
// Added copyright notice.
//
// Revision 1.2  1995/03/14  02:17:25  jimg
// Modified so that the new netio library is used.
// Modified so that C++ Strings are used in place of C strings.
// Removed old code copied from the dods netcdf sources.
//
// Revision 1.1  1995/01/09  16:04:52  dan
// These files constitute the JGOFS server and client library code
// for use with the DODS api.
//
// Revision 1.1  1994/09/27  23:19:37  jimg
// First version of the netcdf software which used libdas++.a.
//

static char rcsid[]={"$Id: jgdas.cc,v 1.11 1998/02/05 20:14:38 jimg Exp $"};

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <iostream.h>

#if 0
extern "C" {
    int ljdbopen_(int *, char *, char (*)[40], int *, int *);
    int ljdbattributes_(int *, int *, char *);
    int ljdberror_(int *, char (*)[1024]);
};
#endif

#include "DAS.h"
#include "jgofs.h"

// These are used as the return values for print_type().

static const char STRING[]="String";
static const char BYTE[]="Byte";
static const char INT32[]="Int32";
static const char FLOAT64[]="Float64";

// Used by ErrMsgT

static char Msgt[255];
extern void ErrMsgT(const char *Msgt);


// Given a pointer to the jgofs variable id and a pointer to an attribute
// table pointer, read the attributes and store their names and values in the
// attribute table.
//
// NB: currently values are stored only as strings.
//
// Returns: Always returns true.

bool
read_attributes(int *v,  AttrTable *at)
{
    char attrname[TOKEN];
    char tmpStr[TOKEN];
    char quotedStr[TOKEN];
    char *nameP, namePart[64], *valP, valPart[64];
    char *dashToken, dash[64], *equalToken, equal[64];
    char *attr[10];
    char *tmpS;
    int  nattrs;

    nameP = &namePart[0]; valP = &valPart[0];
    dashToken = &dash[0]; equalToken = &equal[0];
    if( ljdbattributes_(&unit, v, attrname) ) {
      nattrs = 0;
      attr[nattrs] = (char *)malloc(sizeof(char[128]));
      attr[nattrs] = strtok(attrname,";");
      
      while ( attr[nattrs] != NULL && nattrs < 10 ) 
	{
	  nattrs++;
	  attr[nattrs] = (char *)malloc(sizeof(char[128]));
	  attr[nattrs] = strtok(NULL,";");
	}

      for ( int i = 0; i < nattrs; ++i ) {
	dashToken = strchr( attr[i], '-');
	equalToken = strchr( attr[i], '=');

	if ( !dashToken && !equalToken ) {
	  sprintf(quotedStr, "\"%s\"", attr[i]);
	  at->append_attr("units", "STRING", quotedStr);
	}
	else {
	  if ( equalToken ) {
	    valP = strcpy( valP, equalToken+1);
	    *equalToken = '\0';
	    sprintf(quotedStr, "\"%s\"", valP);
	    at->append_attr(attr[i], "STRING", quotedStr);
	  }
	  else {
	    if ( dashToken ) {
	      valP = strcpy( valP, dashToken+1);
	      *dashToken = '\0';
	      sprintf(quotedStr, "\"%s\"", valP);
	      at->append_attr(attr[i], "STRING", quotedStr);
	    }
	  }
	}
      }
    }
    
    return true;
}

// Given a reference to an instance of class DAS and a filename that referrs
// to a JGOFS object, read the JGOFS object and extract all the attributes
// of each of its variables. Add the variables and their attributes to the
// instance of DAS.
//
// Returns: false if an error accessing the JGOFS object was detected, true
// otherwise. 

bool
create_das(DAS &das, char *filename, String *error)
{
    bool status;
    char msg[1024];

    // Replaced -99 with -NVALS. jhrg 11/8/96
    nVars = -(NVALS);
    int nLvls = ljdbopen_(&unit, filename, varName, &varSize, &nVars);
    AttrTable *attr_table_ptr;

    if (nLvls < 0) {
        if ( status = ljdberror_(&unit, &msg) )
	  *(error) +=  (String)"\"Could not open " + (String)filename + (String)": "
	            +  (String)msg + (String)"\"";
	else 
	  *(error) += (String)"\"JGOFS could not open object " 
	            + (String)filename + (String)"\"";
	return false;
    }

    // `v' is the variable id passed to read_attributes().

    for (int v = 0; v < nVars; ++v) {
	attr_table_ptr = das.get_table(varName[v]);
	if (!attr_table_ptr)
	    attr_table_ptr = das.add_table(varName[v], new AttrTable);
      
	if (!read_attributes(&v, attr_table_ptr))
	    return false;
    }

    return true;
}

